@@ -47,7 +47,7 @@ def add(i):
 
               (cids[0])                  - as uoa or full CID
 
-              (path)                     - if !='' - creat in this path
+              (path)                     - if !='' - create in this path or import from this path
               (here)                     - if =='yes', create in current path
               (use_default_path)         - if 'yes' create repository in the default path (CK_REPOS)
                                            instead of the current path (default is 'yes')
@@ -72,6 +72,11 @@ def add(i):
                                            (useful when kernel is set to allow writing only to such repositories)
 
               (url)                      - if type=='remote' or 'git', URL of remote repository or git repository
+              (hostname)                 - if !='', automatically form url above (add http:// + /ck?)
+              (port)                     - if !='', automatically add to url above
+              (hostext)                  - if !='', add to the end of above URL instead of '/ck?' -
+                                           useful when CK server is accessed via Apache2, IIS, Nginx or other web servers
+
               (githubuser)               - if shared repo, use this GitHub user space instead of default "ctuning"
               (sync)                     - if 'yes' and type=='git', sync repo after each write operation
 
@@ -89,10 +94,13 @@ def add(i):
               (skip_reusing_remote_info) - if 'yes', do not reuse remote .cmr.json description of a repository
 
               (current_repos)            - if resolving dependencies on other repos, list of repos being updated (to avoid infinite recursion)
+
+              (describe)                 - describe repository for Artifact Evaluation (see http://cTuning.org/ae)
             }
 
     Output: {
               return       - return code =  0, if successful
+                                           16, repository with a given path is already registered in CK
                                          >  0, if error
               (error)      - error text if return > 0
             }
@@ -134,7 +142,10 @@ def add(i):
     share=i.get('share','')
     if share=='yes' and shared=='': shared='git'
 
-    url=i.get('url','')
+    rx=form_url(i)
+    if rx['return']>0: return rx
+    url=rx['url']
+
     sync=i.get('sync','')
     df=i.get('default','')
 
@@ -289,7 +300,7 @@ def add(i):
     # Check if repository is already registered with this path
     r=ck.find_repo_by_path({'path':p})
     if r['return']==0:
-       return {'return':1, 'error':'repository with a given path is already registered in CK'}
+       return {'return':16, 'error':'repository with a given path is already registered in CK'}
     elif r['return']!=16: 
        return r
 
@@ -307,6 +318,7 @@ def add(i):
 
     # If git, clone repo
     repo_had_local=True
+    dd={}
     if remote!='yes' and shared=='git':
        r=pull({'path':p, 'type':shared, 'url':url, 'clone':'yes', 'git':i.get('git',''), 'out':o})
        if r['return']>0: return r
@@ -316,6 +328,8 @@ def add(i):
           r=ck.load_json_file({'json_file':py})
           if r['return']>0: return r
           dc=r['dict']
+          ddc=dc.get('dict',{})
+          dd.update(ddc)
 
           xd=dc.get('data_uoa','')
           xdi=dc.get('data_uid','')
@@ -335,7 +349,6 @@ def add(i):
           repo_had_local=False
 
     # Prepare meta description
-    dd={}
     if df=='yes': 
        dd['default']='yes'
     if remote=='yes': 
@@ -355,6 +368,11 @@ def add(i):
     if len(rdeps)>0:
        dd['repo_deps']=rdeps
 
+    # Check if need to describe for Artifact Evaluation
+    if i.get('describe','')=='yes':
+       r=describe({'dict':dd})
+       if r['return']>0: return r
+
     # If not default, go to common core function to create entry
     if df!='yes':
        ii={'module_uoa':work['self_module_uoa'],
@@ -462,6 +480,11 @@ def update(i):
               (shared)                   - if not remote and =='git', shared through GIT
 
               (url)                      - if type=='git', URL of remote repository or git repository
+              (hostname)                 - if !='', automatically form url above (add http:// + /ck?)
+              (port)                     - if !='', automatically add to url above
+              (hostext)                  - if !='', add to the end of above URL instead of '/ck?' -
+                                           useful when CK server is accessed via Apache2, IIS, Nginx or other web servers
+
               (sync)                     - if 'yes' and type=='git', sync repo after each write operation
               (allow_writing)            - if 'yes', allow writing 
                                            (useful when kernel is set to allow writing only to such repositories)
@@ -472,6 +495,8 @@ def update(i):
                                              ("repo_url") - URL of the shared repository (if not from github.com/ctuning)
 
               (update)                   - if 'yes', force updating
+
+              (describe)                 - describe repository for Artifact Evaluation (see http://cTuning.org/ae)
             }
 
     Output: {
@@ -493,9 +518,12 @@ def update(i):
     remote=i.get('remote','')
     rruoa=i.get('remote_repo_uoa','')
     shared=i.get('shared','')
-    url=i.get('url','')
     sync=i.get('sync','')
 
+    rx=form_url(i)
+    if rx['return']>0: return rx
+    url=rx['url']
+
     rdeps=i.get('repo_deps',[])
 
     eaw=i.get('allow_writing','')
@@ -599,6 +627,12 @@ def update(i):
               d['repo_deps'].append(q)
        changed=True
 
+    # Check if need to describe for Artifact Evaluation
+    if i.get('describe','')=='yes':
+       r=describe({'dict':d})
+       if r['return']>0: return r
+       changed=True
+
     # Write if changed
     if changed or i.get('update','')=='yes':
        if o=='con':
@@ -669,6 +703,8 @@ def pull(i):
 
     o=i.get('out','')
 
+    xrecache=False
+
     pp=[]
     px=i.get('path','')
     t=i.get('type','')
@@ -683,10 +719,23 @@ def pull(i):
        pp.append({'path':px, 'type':t, 'url':url})
 
     uoa=i.get('data_uoa','')
-    cids=i.get('cids',[])
-    if len(cids)>0: uoa=cids[0]
 
-    if uoa=='' and len(pp)==0: uoa='*'
+    cids=i.get('cids',[])
+    if len(cids)>0 and uoa=='': 
+       uoa=cids[0]
+
+    # If url is not empty and uoa is empty, get name from URL:
+    if url!='' and uoa=='' and px=='':
+       ix=url.rfind('/')
+       if ix>0:
+          uoa=url[ix+1:]
+          if uoa.endswith('.git'):
+             uoa=uoa[:-4]
+
+          i['data_uoa']=uoa
+
+    if uoa=='' and len(pp)==0 and url=='': 
+       uoa='*'
 
     if uoa!='':
        if uoa.find('*')>=0 or uoa.find('?')>=0:
@@ -705,6 +754,9 @@ def pull(i):
               t=d.get('shared','')
               duoa=r['data_uoa']
 
+              if d.get('recache','')=='yes':
+                 xrecache=True
+
               if t!='':
                  p=d.get('path','')
                  url=d.get('url','')
@@ -733,6 +785,9 @@ def pull(i):
           d=r['dict']
           duoa=r['data_uoa']
 
+          if d.get('recache','')=='yes':
+             xrecache=True
+
           p=d['path']
           t=d.get('shared','')
           url=d.get('url','')
@@ -759,6 +814,7 @@ def pull(i):
         if o=='con' and tt!='clone':
            ck.out('******************************************************************')
            ck.out('Updating repo "'+duoa+'" ...')
+           ck.out('')
            ck.out('  Local path: '+p)
            ck.out('  URL:        '+url)
 
@@ -831,6 +887,16 @@ def pull(i):
                    'out':o})
            if r['return']>0: return r
 
+    # Re-caching ...
+    if xrecache:
+       if o=='con':
+          ck.out('  ==============================================')
+          ck.out('  At least one repository requires recaching ...')
+          ck.out('')
+
+          r=recache({'out':o})
+          if r['return']>0: return r
+
     return {'return':0}
 
 ##############################################################################
@@ -1863,7 +1929,6 @@ def renew(i):
     """
     Input:  {
               data_uoa                   - data UOA of the repo
-
             }
 
     Output: {
@@ -1918,3 +1983,177 @@ def renew(i):
         'data_name':dn,
         'url':url}
     return ck.access(ii)
+
+##############################################################################
+# show remote repo in a browser
+
+def show(i):
+    """
+    Input:  {
+              data_uoa                   - data UOA of the repo
+            }
+
+    Output: {
+              return       - return code =  0, if successful
+                                         >  0, if error
+              (error)      - error text if return > 0
+            }
+
+    """
+
+    o=i.get('out','')
+
+    duoa=i.get('data_uoa','')
+    if duoa=='':
+       return {'return':1, 'error':'repository UOA is not specified'}
+
+    # Get configuration (not from Cache - can be outdated info!)
+#    r=ck.load_repo_info_from_cache({'repo_uoa':duoa})
+    r=ck.access({'action':'load',
+                 'module_uoa':work['self_module_uoa'],
+                 'data_uoa':duoa})
+    if r['return']>0: return r
+
+    p=r.get('dict',{}).get('path','')
+    d=r['dict']
+    dn=r.get('data_name','')
+
+    shared=d.get('shared','')
+    url=d.get('url','')
+
+    if shared!='git' and url=='':
+       return {'return':1, 'error':'this repository is not shared!'}
+
+    import webbrowser
+    webbrowser.open(url)
+
+    return {'return':0}
+
+##############################################################################
+# describe repository for Artifact Evaluation
+
+def describe(i):
+    """
+    Input:  {
+              (dict)  - dict with current repo description
+            }
+
+    Output: {
+              return       - return code =  0, if successful
+                                         >  0, if error
+              (error)      - error text if return > 0
+
+              (dict)  - updated dict with current repo description
+            }
+
+    """
+
+    d=i.get('dict',{})
+
+    a=d.get('artifact_pack_description',{})
+
+    ck.out('')
+    ck.out('Please, provide description of your artifact pack (see http://cTuning.org/ae for more details):')
+    ck.out('')
+
+    r=ck.inp({'text':'Enter title of your artifact pack (or associated paper): '})
+    x=r['string']
+    if x!='': a['title']=x
+
+    r=ck.inp({'text':'List contributors: '})
+    x=r['string']
+    if x!='': a['contributors']=x
+
+    r=ck.inp({'text':'Enter artifact abstract: '})
+    x=r['string']
+    if x!='': a['description']=x
+
+    r=ck.inp({'text':'Enter license of the whole artifact pack: '})
+    x=r['string']
+    if x!='': a['license']=x
+
+    r=ck.inp({'text':'Enter version/revision of the whole artifact pack: '})
+    x=r['string']
+    if x!='': a['version']=x
+
+    r=ck.inp({'text':'How delivered (URL, DOI, OCRID, etc): '})
+    x=r['string']
+    if x!='': a['how_delivered']=x
+
+    r=ck.inp({'text':'Describe software dependencies: '})
+    x=r['string']
+    if x!='': a['software_dependencies']=x
+
+    r=ck.inp({'text':'Describe hardware dependencies: '})
+    x=r['string']
+    if x!='': a['hardware_dependencies']=x
+
+    r=ck.inp({'text':'Describe data sets: '})
+    x=r['string']
+    if x!='': a['datasets']=x
+
+    r=ck.inp({'text':'Describe installation procedure: '})
+    x=r['string']
+    if x!='': a['installation']=x
+
+    r=ck.inp({'text':'Describe possible experiment parameterization: '})
+    x=r['string']
+    if x!='': a['parameterization']=x
+
+    r=ck.inp({'text':'Describe experiment workflow: '})
+    x=r['string']
+    if x!='': a['experiment_workflow']=x
+
+    r=ck.inp({'text':'Describe evaluation procedure and expected output: '})
+    x=r['string']
+    if x!='': a['evaluation']=x
+
+    r=ck.inp({'text':'Acknowledgments: '})
+    x=r['string']
+    if x!='': a['acknowledgments']=x
+
+    r=ck.inp({'text':'Misc notes: '})
+    x=r['string']
+    if x!='': a['notes']=x
+
+    d['artifact_pack_description']=a
+
+    return {'return':0, 'dict':d}
+
+##############################################################################
+# form URL from hostname, hostext and port
+
+def form_url(i):
+    """
+    Input:  {
+              (url)                      - if type=='remote' or 'git', URL of remote repository or git repository
+              (hostname)                 - if !='', automatically form url above (add http:// + /ck?)
+              (port)                     - if !='', automatically add to url above
+              (hostext)                  - if !='', add to the end of above URL instead of '/ck?' -
+                                           useful when CK server is accessed via Apache2, IIS or other web servers
+            }
+
+    Output: {
+              return       - return code =  0, if successful
+                                         >  0, if error
+              (error)      - error text if return > 0
+
+              url          - formed URL
+            }
+
+    """
+
+    url=i.get('url','')
+
+    if i.get('hostname','')!='':
+       url='http://'+i['hostname']
+
+       if i.get('port','')!='':
+          url+=':'+i['port']
+
+       if i.get('hostext','')!='':
+          url+='/'+i['hostext']
+       else:
+          url+='/ck?'
+
+    return {'return':0, 'url':url}
